
import saga as rs


def test_attributes():
    class Description (rs.Attributes) :

        def __init__(self, seed):

            import saga.attributes as rsa

            rs.Attributes.__init__ (self)

            self._attributes_extensible    (False)
            self._attributes_allow_private (True)
            self._attributes_camelcasing   (True)

            # register properties with the attribute interface

            self._attributes_register  ('_config'                   , None, rsa.STRING, rsa.SCALAR, rsa.WRITEABLE)
            self._attributes_register  (rs.job.EXECUTABLE           , None, rsa.STRING, rsa.SCALAR, rsa.WRITEABLE)
            self._attributes_register  (rs.job.INTERACTIVE          , None, rsa.BOOL,   rsa.SCALAR, rsa.WRITEABLE)
            self._attributes_register  (rs.job.CLEANUP              , None, rsa.BOOL,   rsa.SCALAR, rsa.WRITEABLE)
            self._attributes_register  (rs.job.JOB_START_TIME       , None, rsa.TIME,   rsa.SCALAR, rsa.WRITEABLE)
            self._attributes_register  (rs.job.ENVIRONMENT          , None, rsa.STRING, rsa.VECTOR, rsa.WRITEABLE)
            self._attributes_register  (rs.job.TOTAL_CPU_COUNT      , None, rsa.INT,    rsa.SCALAR, rsa.WRITEABLE)
            self._attributes_register  (rs.job.SPMD_VARIATION       , None, rsa.ENUM,   rsa.SCALAR, rsa.WRITEABLE)
            self._attributes_set_enums (rs.job.SPMD_VARIATION,      ['MPI', 'OpenMP', 'MPICH-G', None])

            self._env_is_list = False

            self._attributes_set_getter (rs.job.ENVIRONMENT, self._get_env)
            self._attributes_set_setter (rs.job.ENVIRONMENT, self._set_env)

            self.from_dict(seed)


        # --------------------------------------------------------------------------
        #
        def _set_env (self, val) :
            if  isinstance (val, list) :
                self._env_is_list = True


        # --------------------------------------------------------------------------
        #
        def _get_env (self) :
            env = self.get_attribute (rs.job.ENVIRONMENT)
            if  self._env_is_list :
                self._env_is_list = False
                return ["%s=%s" % (key, val) for (key, val) in env.items ()]
            return env


    k_str = rs.job.EXECUTABLE
    v_str = '/bin/date'
    k_int = rs.job.TOTAL_CPU_COUNT
    v_int = 2
    k_inv = 'invalid'
    v_inv = True

    seed_1 = {k_str: v_str,
              k_int: v_int,
              k_inv: v_inv}
    seed_2 = {k_str: v_str,
              k_int: v_int}

    try :
        d = Description(seed_1)
        failed = False
    except rs.exceptions.IncorrectState:
        failed = True
    assert(failed)

    d = Description(seed_2)
    assert(d)

    assert(k_str in d.as_dict())
    assert(k_int in d.as_dict())

    assert(d[k_str] == v_str)
    assert(d[k_int] == v_int)

    assert(isinstance(d[k_str], str))
    assert(isinstance(d[k_int], int))


# ------------------------------------------------------------------------------
#
if __name__ == '__main__':

    test_attributes()


# ------------------------------------------------------------------------------

